;;; emacspeak-websearch.el --- search utilities
;;; $Id: emacspeak-websearch.el,v 10.0 1999/04/30 17:36:56 raman Exp $
;;; $Author: raman $
;;; Description:  Emacspeak extension to make Web searching convenient
;;; Keywords: Emacspeak, WWW interaction
;;{{{  LCD Archive entry:

;;; LCD Archive Entry:
;;; emacspeak| T. V. Raman |raman@adobe.com
;;; A speech interface to Emacs |
;;; $Date: 1999/04/30 17:36:56 $ |
;;;  $Revision: 10.0 $ |
;;; Location undetermined
;;;

;;}}}
;;{{{  Copyright:
;;;Copyright (C) 1995, 1996, 1997, 1998, 1999   T. V. Raman  Adobe Systems Incorporated
;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
;;; All Rights Reserved.
;;;
;;; This file is not part of GNU Emacs, but the same permissions apply.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;}}}
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;{{{ required modules

(require 'cl)
(declaim  (optimize  (safety 0) (speed 3)))
(require 'advice)
(require 'emacspeak-speak)
(require 'thingatpt)
(require 'voice-lock)
(require 'emacspeak-fix-interactive)
(require 'emacspeak-sounds)
(require 'webjump)
(require 'browse-url)

;;}}}
;;{{{  Introduction:

;;; Commentary:

;;; This module provides utility functions for searching the WWW

;;; Code:

;;}}}
;;{{{ let's use W3 if it is available:

(eval-when (load)
  (declaim (special browse-url-browser-function))
  (when (locate-library "w3")
    (setq browse-url-browser-function 
          'browse-url-w3)))

;;}}}
;;{{{ searcher table

(defvar emacspeak-websearch-table (make-hash-table)
  "Table holding mapping from search engine names to appropriate searcher functions.")

(defsubst emacspeak-websearch-set-searcher  (engine searcher)
  (declare (special emacspeak-websearch-table))
  (setf (gethash engine emacspeak-websearch-table) searcher))

(defsubst emacspeak-websearch-get-searcher (engine)
  (declare (special emacspeak-websearch-table))
  (gethash engine emacspeak-websearch-table))

;;}}}
;;{{{ set up engines 

(emacspeak-websearch-set-searcher 'bbc
                                  'emacspeak-websearch-bbc-search)
(emacspeak-websearch-set-searcher 'britannica
                                  'emacspeak-websearch-britannica-search)
(emacspeak-websearch-set-searcher 'cnn
                                  'emacspeak-websearch-cnn-search)

(emacspeak-websearch-set-searcher 'dejanews
                                  'emacspeak-websearch-dejanews-search)
(emacspeak-websearch-set-searcher 'merriam-webster
                                  'emacspeak-websearch-merriam-webster-search)
(emacspeak-websearch-set-searcher 'dictionary-hypertext-webster
                                  'emacspeak-websearch-dictionary-hypertext-webster-search)
(emacspeak-websearch-set-searcher 'usenet-altavista
                                  'emacspeak-websearch-usenet-altavista-search)
(emacspeak-websearch-set-searcher 'altavista
                                  'emacspeak-websearch-altavista-search)

(emacspeak-websearch-set-searcher 'google
                                  'emacspeak-websearch-google)

(emacspeak-websearch-set-searcher 'hotbot
                                  'emacspeak-websearch-hotbot-search)


(emacspeak-websearch-set-searcher 'inference
                                  'emacspeak-websearch-inference-search)
(emacspeak-websearch-set-searcher 'news-yahoo
                                  'emacspeak-websearch-news-yahoo)
(emacspeak-websearch-set-searcher 'map-directions
                                  'emacspeak-websearch-map-directions-search)
(emacspeak-websearch-set-searcher 'yahoo 'emacspeak-websearch-yahoo)

(emacspeak-websearch-set-searcher 'weather
                                  'emacspeak-websearch-weather)

(emacspeak-websearch-set-searcher 'w3c
                                  'emacspeak-websearch-w3c-search)

(emacspeak-websearch-set-searcher 'rfb
                                  'emacspeak-websearch-rfb-search)

;;}}}
;;{{{ top-level dispatch 

(defun emacspeak-websearch-dispatch  ()
  "Launches specific websearch queries.
Search engine is selected by keystrokes listed below.
Once selected, the selected searcher prompts for additional
information as appropriate.
Key     Engine              Search
a       AltaVista        AltaVista Simple Search
b       BBC              BBC Archives
c       CNN              CNN Interactive
d       dejanews         Usenet Archives At Dejanews
Cap D   Dictionary  Hypertext Websters Dictionary
e   Encyclopedia                  Encyclopedia Britannica
g       Google           The Google WWW Index
h       Hotbot           The HotBot WWW Index
i       Inference        The Inference WWW Search 
m       Map           Driving directions from Yahoo
Cap M   Dictionary  Merriam Webster Dictionary
n       news-yahoo       News Wire at Yahoo
Cap R   RFBD  RFB&D Catalog search
u       usenet-altavista Usenet Index At AltaVista
w       Weather           Weather Channel  By Zip Code
cap W   W3C               Search W3C Site
y       yahoo            Master Yahoo Index
When using W3,  this interface attempts to speak the most relevant information on the result page."
  (interactive)
  (let ((engine nil)
        (searcher nil)
        (e nil))
    (while (null engine)
      (setq e (read-char "Search Engine: "))
      (case e
        (?a (setq engine 'altavista))
        (?b (setq engine 'bbc))
        (?e (setq engine 'britannica))
        (?c (setq engine 'cnn))
        (?g (setq engine 'google))
        (?h (setq engine 'hotbot))
        (?i (setq engine 'inference))
        (?n (setq engine 'news-yahoo))
        (?m (setq engine 'map-directions))
        (?M (setq engine 'merriam-webster))
        (?R (setq engine 'rfb))
        (?y (setq engine 'yahoo))
        (?u (setq engine 'usenet-altavista))
        (?d (setq engine 'dejanews))
            (?D (setq engine 'dictionary-hypertext-webster))
        (?w (setq engine 'weather))
        (?W (setq engine 'w3c))
        (otherwise  (progn 
                      (describe-function this-command)
                      (emacspeak-speak-help)
                      (sit-for 180)))))
    (setq searcher
          (emacspeak-websearch-get-searcher engine))
    (if searcher 
        (call-interactively searcher)
      (error "I do not know how to search using %s" engine))))

;;}}}
;;{{{ websearch utilities

;;{{{ helpers to read the query 

(defvar emacspeak-websearch-history nil
  "Holds history of search queries.")

(defsubst emacspeak-websearch-read-query (prompt)
  (let ((answer 
         (read-from-minibuffer 
          prompt
          nil nil nil
          (car emacspeak-websearch-history)
          (word-at-point))))
    (pushnew answer  emacspeak-websearch-history :test
             #'string-equal)
    answer))

;;}}}
;;{{{ post processer hook

(defsubst emacspeak-websearch-post-process (locator speaker &rest args)
  "Post processing steps on a result page.
LOCATOR is a string to search for in the results page.
SPEAKER is a function to call to speak relevant information.
ARGS specifies additional arguments to SPEAKER if any."
  (when (eq browse-url-browser-function 'browse-url-w3)
    (cond
     ((search-forward locator nil t)
      (recenter 0)
      (apply speaker args))
     (t (message "Your search appears to have ffailed.")))))

;;}}}
;;{{{ google

(defvar emacspeak-websearch-google-uri
  "http://www.google.com/search?q="
  "*URI for Google search")

(defun emacspeak-websearch-google (query)
  "Perform an Google search"
  (interactive
   (list (emacspeak-websearch-read-query "Google Query: ")))
  (declare (special emacspeak-websearch-google-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-google-uri
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "results"
   'emacspeak-speak-line))

;;}}}
;;{{{ yahoo

(defvar emacspeak-websearch-yahoo-uri
  "http://search.yahoo.com/bin/search?p="
  "*URI for launching a Yahoo  search")

(defun emacspeak-websearch-yahoo (query)
  "Perform an Yahoo  search"
  (interactive
   (list (emacspeak-websearch-read-query "Yahoo Query: ")))
  (declare (special emacspeak-websearch-yahoo-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-yahoo-uri
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "Web Pages"
   'emacspeak-speak-line))

;;}}}
;;{{{  news yahoo

(defvar emacspeak-websearch-news-yahoo-uri
  "http://search.news.yahoo.com/search/news?n=30&p="
  "*URI for launching a Yahoo News search")

(defun emacspeak-websearch-news-yahoo (query)
  "Perform an Yahoo News search"
  (interactive
   (list (emacspeak-websearch-read-query "Yahoo News Query: ")))
  (declare (special emacspeak-websearch-news-yahoo-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-news-yahoo-uri
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "Matches"
  'emacspeak-speak-line))

;;}}}
;;{{{ CNN

(defvar emacspeak-websearch-cnn-uri
  "http://search.cnn.com:8765/query.html?qt="
  "*URI for launching a CNN Interactive  search.")

(defvar emacspeak-websearch-cnn-options
  "&col=cnni&lk=1&rf=1"
"*Additional default options to pass to CNN.")


(defun emacspeak-websearch-cnn-search (query &optional prefix)
  "Perform an CNN search.  
Optional interactive prefix arg
prompts for additional search parameters.  The default is to
sort by date and show summaries.  To sort by relevance
specify additional parameter &rf=0.  To hide summaries,
specify additional parameter &lk=2.
You can customize the defaults by setting variable
emacspeak-websearch-cnn-options to an appropriate string."
  (interactive
   (list
    (emacspeak-websearch-read-query "CNN Interactive Query:
")
    current-prefix-arg))
  (declare (special emacspeak-websearch-cnn-uri
                    emacspeak-websearch-cnn-options))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-cnn-uri
             (webjump-url-encode query)
             emacspeak-websearch-cnn-options
             (if prefix 
                 (read-from-minibuffer
                  "Additional query parameters: ")
               ""))))
  (search-forward "Results")
  (w3-table-focus-on-this-cell)
  (emacspeak-websearch-post-process
   "Results"
  'emacspeak-speak-line))

;;}}}
;;{{{  altavista 

(defvar emacspeak-websearch-altavista-uri 
  "http://www.altavista.com/cgi-bin/query?pg=q&KL=en&wfmt=c&ufmt=d&text=yes&kl=XX&act=search"
  "URI for simple Altavista search")

(defun emacspeak-websearch-altavista-search (query)
  "Perform an Altavista search"
  (interactive
   (list (emacspeak-websearch-read-query "Altavista Query: ")))
  (declare (special emacspeak-websearch-altavista-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-altavista-uri
             "&q="
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "AltaVista found"
  'emacspeak-speak-line))

;;}}}
;;{{{  usenet altavista 

(defvar emacspeak-websearch-usenet-altavista-uri 
  "http://www.altavista.com/cgi-bin/query?pg=q&what=news&KL=en&enc=iso88591&text=yes"
  "URI for Usenet  Altavista search")

(defun emacspeak-websearch-usenet-altavista-search (query)
  "Perform a Usenet  Altavista search"
  (interactive
   (list (emacspeak-websearch-read-query "Usenet Altavista Query: ")))
  (declare (special emacspeak-websearch-usenet-altavista-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-usenet-altavista-uri
             "&q="
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "AltaVista found"
  'emacspeak-speak-line))

;;}}}
;;{{{  dejanews

(defvar emacspeak-websearch-dejanews-uri 
  "http://www.dejanews.com/[ST_rn=ps]/dnquery.xp?ST=PS&defaultOp=AND&DBS=1&format=terse&maxhits=100&LNG=english&QRY="
  "URI for Dejanews search")

(defun emacspeak-websearch-dejanews-search (query &optional prefix)
  "Perform a Dejanews search"
  (interactive
   (list
    (emacspeak-websearch-read-query "Dejanews Query: ")
    current-prefix-arg))
  (declare (special emacspeak-websearch-dejanews-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-dejanews-uri
             (webjump-url-encode query)
             (if prefix 
                 (read-from-minibuffer
                  "Additional query parameters: ")
               ""))))
  (emacspeak-websearch-post-process
   "Messages"
   'w3-table-speak-this-cell))

;;}}}
;;{{{ altavista emacspeak archive

(defvar emacspeak-websearch-emacspeak-archive-uri
  "http://www.cs.vassar.edu/cgi-bin/wwwwais?selection=Emacspeak+E-list+Archive&sorttype=score&useicons=no&maxhits=40"
  "URI to search Emacspeak mail archive at Vassar.")

(defun emacspeak-websearch-emacspeak-archive (query)
  "Search Emacspeak mail archives.
For example to find messages about Redhat at the Emacspeak
archives, type +redhat"
  (interactive
   (list
    (emacspeak-websearch-read-query "Search for messages matching: ")))
  (declare (special emacspeak-websearch-emacspeak-archive-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-emacspeak-archive-uri
             "&keywords="
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "Here is"
  'emacspeak-speak-line))

;;}}}
;;{{{ Weather

(defvar emacspeak-websearch-weather-uri
  "http://www.weather.com/weather/us/zips/"
  "*URI for getting weather forecast.")

(defun emacspeak-websearch-weather (zip)
  "Get weather forecast for specified zip code."
  (interactive
   (list (emacspeak-websearch-read-query "Zip Code: ")))
  (declare (special emacspeak-websearch-weather-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-weather-uri
             zip
".html")))
  (emacspeak-websearch-post-process
   zip
  'w3-table-speak-this-cell))

;;}}}
;;{{{ hotbot

(defvar emacspeak-websearch-hotbot-uri 
"http://www.hotbot.com/?submit=SEARCH&_v=2"
"URI for searching Hotbot.")

(defvar emacspeak-websearch-hotbot-options 
"&SM=MC&DV=30&LG=english&DC=10&DE=2"
"*Search options for Hotbot search.
SM -- specify phrase search, title search etc.
DV -- specify how many days to cover
LG -- specify language 
DC -- Number of hits 
DE -- full description, summary or URLs only")


(defun emacspeak-websearch-hotbot-search (query &optional prefix)
  "Perform a Hotbot  search.  
Optional interactive prefix arg
prompts for additional search parameters.  The default is to
sort by date and show summaries.  To sort by relevance
specify additional parameter &rf=0.  To hide summaries,
specify additional parameter &lk=2.
You can customize the defaults by setting variable
emacspeak-websearch-hotbot-options to an appropriate string."
  (interactive
   (list
    (emacspeak-websearch-read-query "Hotbot Query: ")
    current-prefix-arg))
  (declare (special emacspeak-websearch-hotbot-uri
                    emacspeak-websearch-hotbot-options))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-hotbot-uri
             "&MT="
             (webjump-url-encode query)
             emacspeak-websearch-hotbot-options
             (if prefix 
                 (read-from-minibuffer
                  "Additional query parameters: ")
               ""))))
  (emacspeak-websearch-post-process
   "Web Matches"
  'w3-table-speak-this-cell))

;;}}}
;;{{{  Inference 

(defvar emacspeak-websearch-inference-uri 
  "http://www.infind.com/infind/infind.exe?query="
  "URI for Inference search")

(defun emacspeak-websearch-inference-search (query)
  "Perform an Inference search."
  (interactive
   (list (emacspeak-websearch-read-query "Inference Query: ")))
  (declare (special emacspeak-websearch-inference-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-inference-uri
             (webjump-url-encode query)))))

;;}}}
;;{{{ Driving directions from Yahoo

(defvar emacspeak-websearch-map-directions-uri
  "http://maps.yahoo.com/py/ddResults.py?Pyt=Tmap&doit=1&newname=&newdesc=&Get+Directions=Get+Directions"
  "URI for getting driving directions from Yahoo.")


(defsubst emacspeak-websearch-map-directions-get-locations ()
  "Convenience function for prompting and constructing the route component."
  (concat 
   (format "&newaddr=%s"
           (webjump-url-encode 
            (read-from-minibuffer "Start Address: ")))
   (format "&newcsz=%s"
           (webjump-url-encode
            (read-from-minibuffer "City/State or Zip:")))
   (format "&newtaddr=%s"
           (webjump-url-encode
            (read-from-minibuffer "Destination Address: ")))
   (format "&newtcsz=%s"
           (webjump-url-encode
            (read-from-minibuffer "City/State or Zip:")))))

(defun emacspeak-websearch-map-directions-search (query)
  "Get driving directions from Yahoo."
  (interactive
   (list 
    (emacspeak-websearch-map-directions-get-locations)))
  (declare (special emacspeak-websearch-map-directions-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-map-directions-uri
             query))
    (emacspeak-websearch-post-process
     "Start out"
    'w3-table-speak-current-table-column)))

;;}}}
;;{{{ W3C

(defvar emacspeak-websearch-w3c-search-uri 
"http://search.w3.org/Member/cgi-bin/query?mss=simple&pg=q&what=web&filter=all&fmt=."
"URI for searching the member area of the W3C site.")

(defun emacspeak-websearch-w3c-search (query)
  "Search the W3C Site."
  (interactive
   (list (emacspeak-websearch-read-query "Search W3C site: ")))
  (declare (special emacspeak-websearch-w3c-search-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-w3c-search-uri
             "&q="
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "match"
  'emacspeak-speak-line))

;;}}}
;;{{{ Webster

(defvar emacspeak-websearch-dictionary-hypertext-webster-uri 
"http://work.ucsd.edu:5141/cgi-bin/http_webster?isindex="
"URI for searching the hypertext Webster dictionary.")

(defun emacspeak-websearch-dictionary-hypertext-webster-search (query)
  "Search the Webster Dictionary."
  (interactive
   (list
    (emacspeak-websearch-read-query "Lookup word in Webster:")))
  (declare (special emacspeak-websearch-dictionary-hypertext-webster-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-dictionary-hypertext-webster-uri
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   query
  'emacspeak-speak-line))

;;}}}
;;{{{ Merriam Webster

(defvar emacspeak-websearch-merriam-webster-uri 
"http://www.m-w.com/cgi-bin/dictionary?va="
"URI for searching the Merriam Webster dictionary.")

(defun emacspeak-websearch-merriam-webster-search (query)
  "Search the Merriam Webster Dictionary."
  (interactive
   (list
    (emacspeak-websearch-read-query "Lookup word in Webster:")))
  (declare (special emacspeak-websearch-merriam-webster-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-merriam-webster-uri
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "Main Entry"
  'emacspeak-speak-line))

;;}}}
;;{{{  RFB

(defvar emacspeak-websearch-rfb-uri
"http://www.rfbd.org/catalog/cat_p.htm?indx=1&cnt=-1&sel_ord=copy&publisher=&cright=&grade=&edition=&isbn=&shelf_num="
"URI for searching RFB catalogues.")
(defun emacspeak-websearch-rfb-search (author title)
  "Search RFB&D catalog."
  (interactive
   (list
    (emacspeak-websearch-read-query "Author:")
    (emacspeak-websearch-read-query "Title:")))
  (declare (special emacspeak-websearch-rfb-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-rfb-uri
"&author="
             (webjump-url-encode author)
             "&title="
             (webjump-url-encode title))))
  (emacspeak-websearch-post-process
   "Number"
  'emacspeak-speak-line))

;;}}}
;;{{{ bbc

(defvar emacspeak-websearch-bbc-uri
"http://news2.thdo.bbc.co.uk/scripts/query.idq?CiMaxRecordsPerPage=32&CiScope=%2FLo&TemplateName=query&CiSort=rank[d]&CiRestriction="
"URI to search the BBC archives.")

(defun emacspeak-websearch-bbc-search (query)
  "Search BBC archives."
  (interactive
   (list
    (emacspeak-websearch-read-query "Search BBC for: ")))
  (declare (special emacspeak-websearch-bbc-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-bbc-uri
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "Your Search"
   'w3-table-focus-on-this-cell))

;;}}}
;;{{{ Britannica

(defvar emacspeak-websearch-britannica-uri 
"http://www.eb.com:180/bol/search?type=topic&I3.x=0&I3.y=0&DBase=Articles"
"URI for searching Britannica online.")

(defun emacspeak-websearch-britannica-search (query)
  "Search Encyclopedia Britannica."
  (interactive
   (list
    (emacspeak-websearch-read-query
     "Search Encyclopedia Britannica  for: ")))
  (declare (special emacspeak-websearch-britannica-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-websearch-britannica-uri
"&query="
             (webjump-url-encode query))))
  (emacspeak-websearch-post-process
   "your search"
   'emacspeak-speak-line))

;;}}}

;;}}}
;;{{{ Browse usenet using Dejanews

(defvar emacspeak-dejanews-uri 
"http://x5.dejanews.com/[ST_rn=bg]/topics_bg.xp?search=topic&group="
"URI to open a group on Dejanews.")

(defun emacspeak-dejanews-browse-group (group &optional prefix)
  "Prompt and browse a newsgroup on Dejanews."
  (interactive
   (list
    (read-from-minibuffer "Newsgroup: ")
current-prefix-arg))
  (declare (special emacspeak-dejanews-uri))
  (let ((url-be-asynchronous nil))
    (browse-url 
     (concat emacspeak-dejanews-uri
             (if prefix 
                 (read-from-minibuffer
                  "Additional query parameters: ")
               "")
             (webjump-url-encode group)
))
    (search-forward group)
    (w3-table-speak-this-cell)))

;;}}}
(provide 'emacspeak-websearch)
;;{{{ end of file

;;; local variables:
;;; folded-file: t
;;; byte-compile-dynamic: t
;;; end:

;;}}}
